@@ -1,4 +1,4 @@
-# Copyright 2017 The TensorFlow Authors. All Rights Reserved.
+# Copyright 2019 The TensorFlow Authors. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -31,42 +31,53 @@ from __future__ import print_function
 import logging
 import os
 
+import pkg_resources
+
+from tensorboard.compat import tf
 from tensorboard.plugins import base_plugin
 from tensorboard.plugins.audio import audio_plugin
-from tensorboard.plugins.beholder import beholder_plugin
+from tensorboard.plugins.beholder import beholder_plugin_loader
 from tensorboard.plugins.core import core_plugin
 from tensorboard.plugins.custom_scalar import custom_scalars_plugin
+from tensorboard.plugins.debugger import debugger_plugin_loader
 from tensorboard.plugins.distribution import distributions_plugin
 from tensorboard.plugins.graph import graphs_plugin
-from tensorboard.plugins.debugger import debugger_plugin_loader
 from tensorboard.plugins.histogram import histograms_plugin
+from tensorboard.plugins.hparams import hparams_plugin_loader
 from tensorboard.plugins.image import images_plugin
-from tensorboard.plugins.interactive_inference import interactive_inference_plugin
+from tensorboard.plugins.interactive_inference import (
+    interactive_inference_plugin_loader
+)
 from tensorboard.plugins.pr_curve import pr_curves_plugin
-from tensorboard.plugins.profile import profile_plugin
+from tensorboard.plugins.profile import profile_plugin_loader
 from tensorboard.plugins.projector import projector_plugin
 from tensorboard.plugins.scalar import scalars_plugin
 from tensorboard.plugins.text import text_plugin
+from tensorboard.plugins.mesh import mesh_plugin
 
 
 logger = logging.getLogger(__name__)
 
+# Ordering matters. The order in which these lines appear determines the
+# ordering of tabs in TensorBoard's GUI.
 _PLUGINS = [
     core_plugin.CorePluginLoader(),
-    beholder_plugin.BeholderPlugin,
     scalars_plugin.ScalarsPlugin,
     custom_scalars_plugin.CustomScalarsPlugin,
     images_plugin.ImagesPlugin,
     audio_plugin.AudioPlugin,
+    debugger_plugin_loader.DebuggerPluginLoader(),
     graphs_plugin.GraphsPlugin,
     distributions_plugin.DistributionsPlugin,
     histograms_plugin.HistogramsPlugin,
-    pr_curves_plugin.PrCurvesPlugin,
     projector_plugin.ProjectorPlugin,
     text_plugin.TextPlugin,
-    interactive_inference_plugin.InteractiveInferencePlugin,
-    profile_plugin.ProfilePluginLoader(),
-    debugger_plugin_loader.DebuggerPluginLoader(),
+    pr_curves_plugin.PrCurvesPlugin,
+    profile_plugin_loader.ProfilePluginLoader(),
+    beholder_plugin_loader.BeholderPluginLoader(),
+    interactive_inference_plugin_loader.InteractiveInferencePluginLoader(),
+    hparams_plugin_loader.HParamsPluginLoader(),
+    mesh_plugin.MeshPlugin,
 ]
 
 def get_plugins():
@@ -79,4 +90,24 @@ def get_plugins():
 
   :rtype: list[Union[base_plugin.TBLoader, Type[base_plugin.TBPlugin]]]
   """
+
   return _PLUGINS[:]
+
+
+def get_dynamic_plugins():
+  """Returns a list specifying TensorBoard's dynamically loaded plugins.
+
+  A dynamic TensorBoard plugin is specified using entry_points [1] and it is
+  the robust way to integrate plugins into TensorBoard.
+
+  This list can be passed to the `tensorboard.program.TensorBoard` API.
+
+  Returns:
+    list of base_plugin.TBLoader or base_plugin.TBPlugin.
+
+  [1]: https://packaging.python.org/specifications/entry-points/
+  """
+  return [
+      entry_point.load()
+      for entry_point in pkg_resources.iter_entry_points('tensorboard_plugins')
+  ]
