@@ -22,7 +22,7 @@ import numpy as np
 import tensorflow as tf
 from google.protobuf import json_format
 from six import iteritems
-from six import string_types
+from six import string_types, integer_types
 from six.moves import zip  # pylint: disable=redefined-builtin
 
 from tensorboard.plugins.interactive_inference.utils import common_utils
@@ -79,14 +79,11 @@ class VizParams(object):
       pieces = [token.strip() for token in pattern.split(',')]
       indices = []
       for piece in pieces:
-        try:
-          if '-' in piece:
-            lower, upper = [int(x.strip()) for x in piece.split('-')]
-            indices.extend(range(lower, upper + 1))
-          else:
-            indices.append(int(piece.strip()))
-        except ValueError as e:
-          raise common_utils.InvalidUserInputError(e)
+        if '-' in piece:
+          lower, upper = [int(x.strip()) for x in piece.split('-', 1)]
+          indices.extend(range(lower, upper + 1))
+        else:
+          indices.append(int(piece.strip()))
       return sorted(indices)
 
     self.x_min = to_float_or_none(x_min)
@@ -97,7 +94,12 @@ class VizParams(object):
     # By default, there are no specific user-requested feature indices.
     self.feature_indices = []
     if feature_index_pattern:
-      self.feature_indices = convert_pattern_to_indices(feature_index_pattern)
+      try:
+        self.feature_indices = convert_pattern_to_indices(
+            feature_index_pattern)
+      except ValueError as e:
+        # If the user-requested range is invalid, use the default range.
+        pass
 
 
 class OriginalFeatureList(object):
@@ -153,7 +155,7 @@ class MutantFeatureValue(object):
           'unexpected type: {}'.format(type(original_feature)))
     self.original_feature = original_feature
 
-    if index is not None and not isinstance(index, int):
+    if index is not None and not isinstance(index, integer_types):
       raise ValueError(
           'index should be None or int, but had unexpected type: {}'.format(
               type(index)))
@@ -165,7 +167,7 @@ class ServingBundle(object):
   """Light-weight class for holding info to make the inference request.
 
   Attributes:
-    inference_address: A local address or blade address to send inference
+    inference_address: An address (such as "hostname:port") to send inference
       requests to.
     model_name: The Servo model name.
     model_type: One of ['classification', 'regression'].
@@ -475,7 +477,8 @@ def mutant_charts_for_feature(example_protos, feature_name, serving_bundles,
 
     charts = []
     for serving_bundle in serving_bundles:
-      inference_result_proto = run_inference(mutant_examples, serving_bundle)
+      (inference_result_proto, _) = run_inference(
+        mutant_examples, serving_bundle)
       charts.append(make_json_formatted_for_single_chart(
         mutant_features, inference_result_proto, index_to_mutate))
     return charts
@@ -594,6 +597,7 @@ def make_json_formatted_for_single_chart(mutant_features,
         x_label: value,
         y_label: sum(y_list) / float(len(y_list))
       })
+    list_of_points.sort(key=lambda p: p[x_label])
     return {key: list_of_points}
 
   else:
@@ -607,11 +611,12 @@ def get_example_features(example):
 
 def run_inference_for_inference_results(examples, serving_bundle):
   """Calls servo and wraps the inference results."""
-  inference_result_proto = run_inference(examples, serving_bundle)
+  (inference_result_proto, attributions) = run_inference(
+    examples, serving_bundle)
   inferences = wrap_inference_results(inference_result_proto)
   infer_json = json_format.MessageToJson(
     inferences, including_default_value_fields=True)
-  return json.loads(infer_json)
+  return json.loads(infer_json), attributions
 
 def get_eligible_features(examples, num_mutants):
   """Returns a list of JSON objects for each feature in the examples.
@@ -732,7 +737,9 @@ def run_inference(examples, serving_bundle):
       make the inference request.
 
   Returns:
-    A ClassificationResponse or RegressionResponse proto.
+    A tuple with the first entry being the ClassificationResponse or
+    RegressionResponse proto and the second entry being a list of the
+    attributions for each example, or None if no attributions exist.
   """
   batch_size = 64
   if serving_bundle.estimator and serving_bundle.feature_spec:
@@ -752,11 +759,20 @@ def run_inference(examples, serving_bundle):
     values = []
     for pred in preds:
       values.append(pred[preds_key])
-    return common_utils.convert_prediction_values(values, serving_bundle)
+    return (common_utils.convert_prediction_values(values, serving_bundle),
+            None)
   elif serving_bundle.custom_predict_fn:
     # If custom_predict_fn is provided, pass examples directly for local
     # inference.
     values = serving_bundle.custom_predict_fn(examples)
-    return common_utils.convert_prediction_values(values, serving_bundle)
+    attributions = None
+    # If the custom prediction function returned a dict, then parse out the
+    # prediction scores and the attributions. If it is just a list, then the
+    # results are the prediction results without attributions.
+    if isinstance(values, dict):
+      attributions = values['attributions']
+      values = values['predictions']
+    return (common_utils.convert_prediction_values(values, serving_bundle),
+            attributions)
   else:
-    return platform_utils.call_servo(examples, serving_bundle)
+    return (platform_utils.call_servo(examples, serving_bundle), None)
